import dolphindb as ddb
import dolphindb.settings as keys
import numpy as np
import pandas as pd
from loguru import logger
import time
class DDBfm():

    ddb_config_servers = {
        'local':
            {'host' : '192.168.64.3',
            'username' : 'admin',
            'password' : '123456'},
        'dev':
            {'host' : '192.168.1.88',
            'username' : 'admin',
            'password' : '123456'},
        'prd':
            {'host' : '192.168.1.7',
            'username' : 'admin',
            'password' : '123456'}
    }

    # len: 71
    all_fm_init=['sc', 'v', 'TS', 'MA', 'AP', 'jm', 'bc', 'bb', 'fu', 'IM', 'IF', 'a', 'lu', 'FG', 'cu', 'al', 'IH', 'RS', 'pg', 'CF', 'SF', 'ni', 'hc', 'UR', 'm', 'SR', 'j', 'PF', 'RM', 'T', 'c', 'JR', 'l', 'p', 'sp', 'CY', 'pb', 'TF', 'b', 'eg', 'rb', 'PK', 'sn', 'nr', 'pp', 'CJ', 'eb', 'SA', 'y', 'RI', 'lh', 'jd', 'OI', 'WH', 'ss', 'ru', 'zn', 'fb', 'rr', 'PM', 'au', 'TA', 'ZC', 'IC', 'bu', 'SM', 'wr', 'cs', 'LR', 'ag', 'i']
    

    ddb_hft_dbPath="dfs://hft_futuremarket_ts"
    ddb_hft_dbName="hft_futuremarket_ts"

    ddf_hft_tick_tbname="TickPartitioned"

    ddf_hft_mink_tbname="MinKlinePartitioned"

    ddf_hft_dailydom_tbname="DailyFutureInfoPartitioned"

    def __init__(self, which_server='local', pool=False):
        self.pool=None
        self.sess =None
        self.hft_db=None
        self.which_server=which_server
        self.ddb_config = self.ddb_config_servers[self.which_server]

        if not pool:
            self.sess = ddb.session(self.ddb_config['host'], 8848)
            self.sess.login(self.ddb_config['username'], self.ddb_config['password'])
            if self.sess.existsDatabase(self.ddb_hft_dbPath):
                self.hft_db=self.sess.database(dbPath=self.ddb_hft_dbPath)

        else:
            self.pool = ddb.DBConnectionPool(self.ddb_config['host'], 8848, 12, self.ddb_config['username'], self.ddb_config['password'])

    # def clear_pool(self):
    #     if self.pool:
    #         while not self.pool.isFinished:
    #             logger.info("pool not finished,sleep for 5 seconds.")
    #             time.sleep(5)
    #         print("pool finished")
    #         logger.debug("pool finished")
    #         self.pool.shutDown()

    def close_sess(self):
        if self.pool:
            while not self.pool.isFinished:
                logger.info("pool not finished,sleep for 5 seconds.")
                time.sleep(5)
            # print("pool finished")
            logger.debug("pool finished")
            self.pool.shutDown()
            
        if self.sess:
            self.sess.clearAllCache()
            self.sess.close()

    def drop_dbpath(self,dbPath):
        if self.sess.existsDatabase(dbPath): 
            self.sess.dropDatabase(dbPath)

    def create_hft_database(self):
        dbPath=self.ddb_hft_dbPath
        dbName=self.ddb_hft_dbName

        if self.sess.existsDatabase(dbPath): 
            self.sess.dropDatabase(dbPath)

        db_init = self.sess.database(dbName='db_init', partitionType=keys.VALUE, partitions=self.all_fm_init, dbPath='')
        
        months=np.array(pd.date_range(start='2000-01', end='2050-12', freq="M"), dtype="datetime64[M]")
        logger.debug(f'created months len: {len(months)}')

        db_date = self.sess.database('db_date', partitionType=keys.VALUE, partitions=months, dbPath='')

        if self.sess.existsDatabase(dbPath): 
            self.sess.dropDatabase(dbPath)

        if self.which_server == 'prd':
            self.hft_db = self.sess.database(dbName=dbName, partitionType=keys.COMPO, partitions=[db_date, db_init], dbPath=dbPath, engine='TSDB')
        else:
            self.hft_db = self.sess.database(dbName=dbName, partitionType=keys.COMPO, partitions=[db_date, db_init], dbPath=dbPath, engine='OLAP')
        
        logger.info(f'Created dbName:{dbName} at dbPath:{dbPath} in {self.which_server}.')
    
    def create_hft_table(self, tbName, df):
        t = self.sess.table(data=df, tableAliasName=tbName)
        if self.which_server == 'prd':
            pt = self.hft_db.createPartitionedTable(table=t, tableName=tbName, partitionColumns=['m_nDatetime', 'code_init'],sortColumns=["code","m_nDatetime"],compressMethods={"m_nDatetime":"delta"})
        else:
            pt = self.hft_db.createPartitionedTable(table=t, tableName=tbName, partitionColumns=['m_nDatetime', 'code_init'])
        pt.append(t)

    def append_hft_table(self, tbName, df):
        appender = ddb.tableAppender(tableName=tbName, ddbSession=self.sess,dbPath=self.ddb_hft_dbPath)
        appender.append(df)
        logger.info(f"sucessfully append some df of {df.shape} to {tbName}")

    def append_pool_hft_table(self, tbName, df, sort_col='m_nDatetime'):
        appender = ddb.PartitionedTableAppender(self.ddb_hft_dbPath, tbName, sort_col, self.pool)
        appender.append(df)
        logger.info(f"Appending some df of {df.shape} to {tbName}")

    def create_daily_info_table(self,df):
        tbName=self.ddf_hft_dailydom_tbname
        t = self.sess.table(data=df, tableAliasName=tbName)

        if self.sess.existsTable(self.ddb_hft_dbPath,tbName):
            self.sess.dropTable(self.ddb_hft_dbPath,tbName)

        if self.which_server == 'prd':
            pt = self.hft_db.createPartitionedTable(table=t, tableName=tbName, partitionColumns=['m_nDate', 'code_init'],sortColumns=["code_init","m_nDate"],compressMethods={"m_nDate":"delta"})
        else:
            pt = self.hft_db.createPartitionedTable(table=t, tableName=tbName, partitionColumns=['m_nDate', 'code_init'])
        pt.append(t)

    def search_code_date_in_tb(self,tbName,curr_date,curr_code):
        """
        messy.depracated.
        """
        curr_date_formatted = curr_date[:4]+'.'+curr_date[4:6]+'.'+curr_date[6:]
        # print('?did i split this right')
        # print(curr_date_formatted)
        tb = self.sess.loadTable(dbPath=self.ddb_hft_dbPath, tableName=tbName)
        logger.info(f"Quickly checking if data on {curr_code} {curr_date} exists...") # could do a slow checking of num of data
        try:
            # doing this cuz there's no method to check if a table is empty lol
            cond=f"code=`{curr_code}, m_nDatetime.date()={curr_date_formatted}d"
            cond=f"code_init=`{curr_code[:-4]}, m_nDatetime.date()={curr_date_formatted}d"
            # print(cond)
            df = tb.select('*').where(cond).top(1).toDF()
            if df.empty or df.shape[0]==0:
                # print(df)
                return False
        except:
            return False
        return True

    def load_tb(self,tableName):
        return self.sess.loadTable(dbPath=self.ddb_hft_dbPath, tableName=tableName)

    def get_missing_code_date_in_tb(self,tbName,curr_date,code_list): # code list has to be like all CF 2022-11-11 fixed it
        logger.info(f"Quickly checking if data on {'+'.join(code_list)} {curr_date} exists...") # could do a slow checking of num of data

        curr_date_formatted = curr_date[:4]+'.'+curr_date[4:6]+'.'+curr_date[6:]
        # print('?did i split this right')
        # print(curr_date_formatted)
        # tb = self.sess.loadTable(dbPath=self.ddb_hft_dbPath, tableName=tbName)
        try:
        
            df = self.sess.run(f"""pt=loadTable("{self.ddb_hft_dbPath}","{tbName}");
                select distinct code from pt where m_nDatetime.date()={curr_date_formatted}d and code in ({" ".join(["`"+c for c in code_list])});
                """)
        #     # doing this cuz there's no method to check if a table is empty lol
        #     # cond=f"code=`{curr_code}, m_nDatetime.date()={curr_date_formatted}d"
        #     # cond=f"code_init=`{code_list[0][:-4]}, m_nDatetime.date()={curr_date_formatted}d"

        #     # code_list_str = " or ".join([f'code=`{c}' for c in code_list])
        #     cond=f"""`m_nDatetime.date()={curr_date_formatted}d and code in ({" ".join(["`"+c for c in code_list])})"""

        #     print(cond)
        #     ddb1.run("""select distinct code from pt where m_nDatetime.date()=2022.11.01d and code in (`AP2305 `AP2310)""")
        #     df = tb.select('distinct code').where(cond).toDF()
            
            if df.empty or df.shape[0]==0:
                # print(df)
                return code_list
        except:
            return code_list

        ddb_code_list = df['distinct_code'].to_list()
        # print(ddb_code_list)
        return_code_list=[]
        for c in code_list:
            if c not in ddb_code_list:
                return_code_list.append(c)
            else:
                logger.warning(f'{c} on {curr_date} is recorded')
        return return_code_list
    
    def get_missing_code_init_date_in_tb(self,tbName,curr_date,code_init_list): # code list has to be like all CF 2022-11-11 fixed it
        logger.info(f"Quickly checking if data on {'+'.join(code_init_list)} {curr_date} exists...") # could do a slow checking of num of data

        curr_date_formatted = curr_date[:4]+'.'+curr_date[4:6]+'.'+curr_date[6:]
        try:
        
            df = self.sess.run(f"""pt=loadTable("{self.ddb_hft_dbPath}","{tbName}");
                select distinct code_init from pt where m_nDatetime.date()={curr_date_formatted}d and code_init in ({" ".join(["`"+c for c in code_init_list])});
                """)
            
            if df.empty or df.shape[0]==0:
                return code_init_list
        except:
            return code_init_list

        ddb_code_list = df['distinct_code_init'].to_list()
        return_code_list=[]
        for c in code_init_list:
            if c not in ddb_code_list:
                return_code_list.append(c)
            else:
                logger.warning(f'{c} on {curr_date} is recorded')
        return return_code_list

if __name__ == '__main__':
    pass